/*
 * Copyright (c) 2007 Extreme! Lab, Indiana University. All rights reserved.
 *
 * This software is open source. See the bottom of this file for the license.
 *
 * $Id: KarmaClient.java,v 1.6 2007/06/28 19:21:46 sshirasu Exp $
 */
package edu.indiana.extreme.xbaya.monitor;

import java.net.URI;

import org.xmlpull.infoset.XmlBuilderException;
import org.xmlpull.infoset.XmlElement;

import wsmg.NotificationHandler;
import xsul5.MLogger;
import edu.indiana.extreme.karma.client.ProvenanceActivityReplay;
import edu.indiana.extreme.karma.client.ProvenanceActivityReplay.ActivityRate;
import edu.indiana.extreme.xbaya.XBayaRuntimeException;
import edu.indiana.extreme.xbaya.util.XMLUtil;

/**
 * @author Satoshi Shirasuna
 */
public class KarmaClient implements NotificationHandler {

    /**
     */
    public enum Rate {
        /**
         * ORIGINAL
         */
        ORIGINAL(ActivityRate.Original),
        /**
         * PERIODIC
         */
        PERIODIC(ActivityRate.Periodic),
        /**
         * BATCH
         */
        BATCH(ActivityRate.Batch);

        private Rate(ActivityRate activityRate) {
            this.activityRate = activityRate;
        }

        private ActivityRate activityRate;

        /**
         * @return ActivityRate
         */
        public ProvenanceActivityReplay.ActivityRate activityRate() {
            return this.activityRate;
        }
    }

    private static final MLogger logger = MLogger.getLogger();

    private Monitor monitor;

    private URI kermaURL;

    private URI workflowInstanceID;

    private Rate rate;

    /**
     * Constructs a KermaClient.
     * 
     * @param monitor
     * @param kermaURL
     * @param workflowInstanceID
     * @param rate
     */
    public KarmaClient(Monitor monitor, URI kermaURL, URI workflowInstanceID,
            Rate rate) {
        this.monitor = monitor;
        this.kermaURL = kermaURL;
        this.workflowInstanceID = workflowInstanceID;
        this.rate = rate;
    }

    /**
     * This method blocks until we get the first notification message.
     */
    public void start() {
        boolean succeed = ProvenanceActivityReplay.replayActivity(this.kermaURL
                .toString(), this.workflowInstanceID.toString(), this.rate
                .activityRate(), this);
        if (!succeed) {
            String message = "Something went wrong inside of the karma library.";
            throw new XBayaRuntimeException(message);
        }
    }

    /**
     * @see wsmg.NotificationHandler#handleNotification(java.lang.String)
     */
    public void handleNotification(String message) {
        try {
            // No SOAP header is included.
            // String soapBody = WsmgUtil.getSoapBodyContent(message);
            // XmlElement event = XMLUtil.stringToXmlElement(soapBody);
            XmlElement event = XMLUtil.stringToXmlElement(message);
            this.monitor.handleNotification(event);
        } catch (XmlBuilderException e) {
            // Just log them because they can be unrelated messages sent to
            // this topic by accident.
            logger.warning("Could not parse received notification: " + message,
                    e);
        } catch (RuntimeException e) {
            logger.warning("Failed to process notification: " + message, e);
        }
    }
}

/*
 * Indiana University Extreme! Lab Software License, Version 1.2
 * 
 * Copyright (c) 2007 The Trustees of Indiana University. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 * 
 * 2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 * 
 * 3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 * 
 * "This product includes software developed by the Indiana University Extreme!
 * Lab. For further information please visit http://www.extreme.indiana.edu/"
 * 
 * Alternatively, this acknowledgment may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 * 
 * 4) The name "Indiana University" or "Indiana University Extreme! Lab" shall
 * not be used to endorse or promote products derived from this software without
 * prior written permission from Indiana University. For written permission,
 * please contact http://www.extreme.indiana.edu/.
 * 
 * 5) Products derived from this software may not use "Indiana University" name
 * nor may "Indiana University" appear in their name, without prior written
 * permission of the Indiana University.
 * 
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of any
 * other entity. Indiana University disclaims any liability to any recipient for
 * claims brought by any other entity based on infringement of intellectual
 * property rights or otherwise.
 * 
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 * INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 * GENERATED USING SOFTWARE.
 */
